@@ -2,18 +2,21 @@
 import ast
 import os
 import os.path
+
 import pandas as pd
+
 import audformat.utils
+
 from nkululeko.data.dataset import Dataset
 import nkululeko.glob_conf as glob_conf
 from nkululeko.reporting.report_item import ReportItem
 
 
 class Dataset_CSV(Dataset):
-    """Class to represent datasets stored as a csv file"""
+    """Class to represent datasets stored as a csv file."""
 
     def load(self):
-        """Load the dataframe with files, speakers and task labels"""
+        """Load the dataframe with files, speakers and task labels."""
         self.util.debug(f"loading {self.name}")
         self.got_target, self.got_speaker, self.got_gender = False, False, False
         data_file = self.util.config_val_data(self.name, "", "")
@@ -37,6 +40,8 @@ class Dataset_CSV(Dataset):
             df = audformat.utils.read_csv(data_file)
         if isinstance(df, pd.Series):
             df = df.to_frame()
+        elif isinstance(df, pd.Index):
+            df = pd.DataFrame(index=df)
         rename_cols = self.util.config_val_data(self.name, "colnames", False)
         if rename_cols:
             col_dict = ast.literal_eval(rename_cols)
@@ -49,7 +54,7 @@ class Dataset_CSV(Dataset):
             if audformat.index_type(df.index) == "segmented":
                 file_index = (
                     df.index.levels[0]
-                    .map(lambda x: root + "/" + audio_path + "/" + x)
+                    .map(lambda x: os.path.join(root, audio_path, x))
                     .values
                 )
                 df = df.set_index(df.index.set_levels(file_index, level="file"))
@@ -58,30 +63,34 @@ class Dataset_CSV(Dataset):
                     df = pd.DataFrame(df)
                 df = df.set_index(
                     df.index.to_series().apply(
-                        lambda x: root + "/" + audio_path + "/" + x
+                        lambda x: os.path.join(root, audio_path, x)
                     )
                 )
         else:  # absolute path is True
             if audformat.index_type(df.index) == "segmented":
                 file_index = (
-                    df.index.levels[0].map(lambda x: audio_path + "/" + x).values
+                    df.index.levels[0].map(lambda x: os.path.join(audio_path, x)).values
                 )
                 df = df.set_index(df.index.set_levels(file_index, level="file"))
             else:
                 if not isinstance(df, pd.DataFrame):
                     df = pd.DataFrame(df)
                 df = df.set_index(
-                    df.index.to_series().apply(lambda x: audio_path + "/" + x)
+                    df.index.to_series().apply(lambda x: os.path.join(audio_path, x))
                 )
 
         self.df = df
         self.db = None
-        self.got_target = True
+        target = self.util.config_val("DATA", "target", None)
+        if target is not None:
+            self.got_target = True
+        else:
+            self.got_target = False
         self.is_labeled = self.got_target
         self.start_fresh = eval(self.util.config_val("DATA", "no_reuse", "False"))
         is_index = False
         try:
-            if self.is_labeled and not "class_label" in self.df.columns:
+            if self.is_labeled and "class_label" not in self.df.columns:
                 self.df["class_label"] = self.df[self.target]
         except AttributeError:
             is_index = True
