@@ -1,17 +1,17 @@
-# experiment.py: Main class for an experiment (nkululeko.nkululeko)
+# nkululeko/experiment.py: Main class for an experiment (nkululeko.nkululeko)
 import ast
 import os
 import pickle
 import random
 import time
 
+import audeer
+import audformat
 import numpy as np
 import pandas as pd
 from sklearn.preprocessing import LabelEncoder
 
-import audeer
-import audformat
-
+import nkululeko.glob_conf as glob_conf
 from nkululeko.data.dataset import Dataset
 from nkululeko.data.dataset_csv import Dataset_CSV
 from nkululeko.demo_predictor import Demo_predictor
@@ -19,12 +19,11 @@ from nkululeko.feat_extract.feats_analyser import FeatureAnalyser
 from nkululeko.feature_extractor import FeatureExtractor
 from nkululeko.file_checker import FileChecker
 from nkululeko.filter_data import DataFilter
-import nkululeko.glob_conf as glob_conf
 from nkululeko.plots import Plots
 from nkululeko.reporting.report import Report
 from nkululeko.runmanager import Runmanager
 from nkululeko.scaler import Scaler
-from nkululeko.test_predictor import TestPredictor
+from nkululeko.testing_predictor import TestPredictor
 from nkululeko.utils.util import Util
 
 
@@ -44,6 +43,7 @@ class Experiment:
         audeer.mkdir(self.data_dir)  # create the experiment directory
         self.util = Util("experiment")
         glob_conf.set_util(self.util)
+        self.split3 = eval(self.util.config_val("EXP", "traindevtest", "False"))
         fresh_report = eval(self.util.config_val("REPORT", "fresh", "False"))
         if not fresh_report:
             try:
@@ -76,11 +76,11 @@ class Experiment:
     #     return self.util.get_exp_name()
 
     def set_globals(self, config_obj):
-        """install a config object in the global space"""
+        """Install a config object in the global space."""
         glob_conf.init_config(config_obj)
 
     def load_datasets(self):
-        """Load all databases specified in the configuration and map the labels"""
+        """Load all databases specified in the configuration and map the labels."""
         ds = ast.literal_eval(glob_conf.config["DATA"]["databases"])
         self.datasets = {}
         self.got_speaker, self.got_gender, self.got_age = False, False, False
@@ -101,12 +101,15 @@ class Experiment:
             if data.got_speaker:
                 self.got_speaker = True
             self.datasets.update({d: data})
-        self.target = self.util.config_val("DATA", "target", "emotion")
+        self.target = self.util.config_val("DATA", "target", "none")
         glob_conf.set_target(self.target)
         # print target via debug
         self.util.debug(f"target: {self.target}")
         # print keys/column
         dbs = ",".join(list(self.datasets.keys()))
+        if self.target == "none":
+            self.util.debug(f"loaded databases {dbs}")
+            return
         labels = self.util.config_val("DATA", "labels", False)
         auto_labels = list(next(iter(self.datasets.values())).df[self.target].unique())
         if labels:
@@ -123,7 +126,15 @@ class Experiment:
         # df = pd.read_csv(storage, header=0, index_col=[0,1,2])
         # df.index.set_levels(pd.to_timedelta(df.index.levels[1]), level=1)
         # df.index.set_levels(pd.to_timedelta(df.index.levels[2]), level=2)
-        df = audformat.utils.read_csv(storage)
+        try:
+            df = audformat.utils.read_csv(storage)
+        except ValueError:
+            # split might be empty
+            return pd.DataFrame()
+        if isinstance(df, pd.Series):
+            df = df.to_frame()
+        elif isinstance(df, pd.Index):
+            df = pd.DataFrame(index=df)
         df.is_labeled = True if self.target in df else False
         # print(df.head())
         return df
@@ -176,6 +187,10 @@ class Experiment:
         store = self.util.get_path("store")
         storage_test = f"{store}testdf.csv"
         storage_train = f"{store}traindf.csv"
+        self.df_dev = None
+        self.feats_dev = None
+        if self.split3:
+            storage_dev = f"{store}devdf.csv"
         start_fresh = eval(self.util.config_val("DATA", "no_reuse", "False"))
         if (
             os.path.isfile(storage_train)
@@ -187,11 +202,24 @@ class Experiment:
             )
             self.df_test = self._import_csv(storage_test)
             self.df_train = self._import_csv(storage_train)
+            self.train_empty = True if self.df_train.shape[0] == 0 else False
+            self.test_empty = True if self.df_test.shape[0] == 0 else False
+            if self.split3:
+                self.df_dev = self._import_csv(storage_dev)
+                self.dev_empty = True if self.df_dev.shape[0] == 0 else False
         else:
             self.df_train, self.df_test = pd.DataFrame(), pd.DataFrame()
+            if self.split3:
+                self.df_dev = pd.DataFrame()
+            else:
+                self.df_dev = None
             for d in self.datasets.values():
-                d.split()
-                d.prepare_labels()
+                if self.split3:
+                    d.split_3()
+                else:
+                    d.split()
+                if self.target != "none":
+                    d.prepare_labels()
                 if d.df_train.shape[0] == 0:
                     self.util.debug(f"warn: {d.name} train empty")
                 self.df_train = pd.concat([self.df_train, d.df_train])
@@ -201,19 +229,38 @@ class Experiment:
                     self.util.debug(f"warn: {d.name} test empty")
                 self.df_test = pd.concat([self.df_test, d.df_test])
                 self.util.copy_flags(d, self.df_test)
+                if self.split3:
+                    if d.df_dev.shape[0] == 0:
+                        self.util.debug(f"warn: {d.name} dev empty")
+                    self.df_dev = pd.concat([self.df_dev, d.df_dev])
+                    self.util.copy_flags(d, self.df_dev)
+            self.train_empty = True if self.df_train.shape[0] == 0 else False
+            self.test_empty = True if self.df_test.shape[0] == 0 else False
+            if self.split3:
+                self.dev_empty = True if self.df_dev.shape[0] == 0 else False
             store = self.util.get_path("store")
             storage_test = f"{store}testdf.csv"
             storage_train = f"{store}traindf.csv"
             self.df_test.to_csv(storage_test)
             self.df_train.to_csv(storage_train)
+            if self.split3:
+                storage_dev = f"{store}devdf.csv"
+                self.df_dev.to_csv(storage_dev)
 
+        if self.target == "none":
+            return
         self.util.copy_flags(self, self.df_test)
         self.util.copy_flags(self, self.df_train)
+        if self.split3:
+            self.util.copy_flags(self, self.df_dev)
         # Try data checks
         datachecker = FileChecker(self.df_train)
         self.df_train = datachecker.all_checks()
         datachecker.set_data(self.df_test)
         self.df_test = datachecker.all_checks()
+        if self.split3:
+            datachecker.set_data(self.df_dev)
+            self.df_dev = datachecker.all_checks()
 
         # Check for filters
         filter_sample_selection = self.util.config_val(
@@ -224,6 +271,9 @@ class Experiment:
             self.df_train = datafilter.all_filters()
             datafilter = DataFilter(self.df_test)
             self.df_test = datafilter.all_filters()
+            if self.split3:
+                datafilter = DataFilter(self.df_dev)
+                self.df_dev = datafilter.all_filters()
         elif filter_sample_selection == "train":
             datafilter = DataFilter(self.df_train)
             self.df_train = datafilter.all_filters()
@@ -231,60 +281,76 @@ class Experiment:
             datafilter = DataFilter(self.df_test)
             self.df_test = datafilter.all_filters()
         else:
-            self.util.error(
+            msg = (
                 "unkown filter sample selection specifier"
                 f" {filter_sample_selection}, should be [all | train | test]"
             )
+            self.util.error(msg)
 
         # encode the labels
         if self.util.exp_is_classification():
             datatype = self.util.config_val("DATA", "type", "dummy")
             if datatype == "continuous":
-                # if self.df_test.is_labeled:
-                #     # remember the target in case they get labelencoded later
-                #     self.df_test["class_label"] = self.df_test[self.target]
-                test_cats = self.df_test["class_label"].unique()
-                # else:
-                #     # if there is no target, copy a dummy label
-                #     self.df_test = self._add_random_target(self.df_test)
-                # if self.df_train.is_labeled:
-                #     # remember the target in case they get labelencoded later
-                #     self.df_train["class_label"] = self.df_train[self.target]
-                train_cats = self.df_train["class_label"].unique()
-
+                if not self.test_empty:
+                    test_cats = self.df_test["class_label"].unique()
+                if not self.train_empty:
+                    train_cats = self.df_train["class_label"].unique()
+                if self.split3 and not self.dev_empty:
+                    dev_cats = self.df_dev["class_label"].unique()
             else:
-                if self.df_test.is_labeled:
-                    test_cats = self.df_test[self.target].unique()
-                else:
-                    # if there is no target, copy a dummy label
-                    self.df_test = self._add_random_target(self.df_test).astype("str")
-                train_cats = self.df_train[self.target].unique()
-                # print(f"df_train: {pd.DataFrame(self.df_train[self.target])}")
-                # print(f"train_cats with target {self.target}: {train_cats}")
-            if self.df_test.is_labeled:
-                if type(test_cats) == np.ndarray:
-                    self.util.debug(f"Categories test (nd.array): {test_cats}")
-                else:
-                    self.util.debug(f"Categories test (list): {list(test_cats)}")
-            if type(train_cats) == np.ndarray:
-                self.util.debug(f"Categories train (nd.array): {train_cats}")
-            else:
-                self.util.debug(f"Categories train (list): {list(train_cats)}")
-
+                if not self.test_empty:
+                    if self.df_test.is_labeled:
+                        test_cats = self.df_test[self.target].unique()
+                    else:
+                        # if there is no target, copy a dummy label
+                        self.df_test = self._add_random_target(self.df_test).astype(
+                            "str"
+                        )
+                if not self.train_empty:
+                    train_cats = self.df_train[self.target].unique()
+                if self.split3 and not self.dev_empty:
+                    dev_cats = self.df_dev[self.target].unique()
             # encode the labels as numbers
             self.label_encoder = LabelEncoder()
-            self.df_train[self.target] = self.label_encoder.fit_transform(
-                self.df_train[self.target]
-            )
-            self.df_test[self.target] = self.label_encoder.transform(
-                self.df_test[self.target]
-            )
             glob_conf.set_label_encoder(self.label_encoder)
+            if not self.train_empty:
+                if isinstance(train_cats, np.ndarray):
+                    self.util.debug(f"Categories train (nd.array): {train_cats}")
+                else:
+                    self.util.debug(f"Categories train (list): {list(train_cats)}")
+
+                self.df_train[self.target] = self.label_encoder.fit_transform(
+                    self.df_train[self.target]
+                )
+            if not self.test_empty:
+                if self.df_test.is_labeled:
+                    if isinstance(test_cats, np.ndarray):
+                        self.util.debug(f"Categories test (nd.array): {test_cats}")
+                    else:
+                        self.util.debug(f"Categories test (list): {list(test_cats)}")
+                if not self.train_empty:
+                    self.df_test[self.target] = self.label_encoder.transform(
+                        self.df_test[self.target]
+                    )
+            if self.split3 and not self.dev_empty:
+                if isinstance(dev_cats, np.ndarray):
+                    self.util.debug(f"Categories dev (nd.array): {dev_cats}")
+                else:
+                    self.util.debug(f"Categories dev (list): {list(dev_cats)}")
+                if not self.train_empty:
+                    self.df_dev[self.target] = self.label_encoder.transform(
+                        self.df_dev[self.target]
+                    )
         if self.got_speaker:
+            speakers_train = 0 if self.train_empty else self.df_train.speaker.nunique()
+            speakers_test = 0 if self.test_empty else self.df_test.speaker.nunique()
             self.util.debug(
-                f"{self.df_test.speaker.nunique()} speakers in test and"
-                f" {self.df_train.speaker.nunique()} speakers in train"
+                f"{speakers_test} speakers in test and"
+                f" {speakers_train} speakers in train"
             )
+            if self.split3:
+                speakers_dev = 0 if self.dev_empty else self.df_dev.speaker.nunique()
+                self.util.debug(f"{speakers_dev} speakers in dev")
 
         target_factor = self.util.config_val("DATA", "target_divide_by", False)
         if target_factor:
@@ -292,6 +358,10 @@ class Experiment:
             self.df_train[self.target] = self.df_train[self.target] / float(
                 target_factor
             )
+            if self.split3:
+                self.df_dev[self.target] = self.df_dev[self.target] / float(
+                    target_factor
+                )
             if not self.util.exp_is_classification():
                 self.df_test["class_label"] = self.df_test["class_label"] / float(
                     target_factor
@@ -299,6 +369,17 @@ class Experiment:
                 self.df_train["class_label"] = self.df_train["class_label"] / float(
                     target_factor
                 )
+                if self.split3:
+                    self.df_dev["class_label"] = self.df_dev["class_label"] / float(
+                        target_factor
+                    )
+        if self.split3:
+            shapes = f"{self.df_train.shape}/{self.df_dev.shape}/{self.df_test.shape}"
+            self.util.debug(f"train/dev/test shape: {shapes}")
+        else:
+            self.util.debug(
+                f"train/test shape: {self.df_train.shape}/{self.df_test.shape}"
+            )
 
     def _add_random_target(self, df):
         labels = glob_conf.labels
@@ -309,9 +390,11 @@ class Experiment:
         return df
 
     def plot_distribution(self, df_labels):
-        """Plot the distribution of samples and speaker per target class and biological sex"""
+        """Plot the distribution of samples and speakers.
+
+        Per target class and biological sex.
+        """
         plot = Plots()
-        sample_selection = self.util.config_val("EXPL", "sample_selection", "all")
         plot.plot_distributions(df_labels)
         if self.got_speaker:
             plot.plot_distributions_speaker(df_labels)
@@ -335,8 +418,16 @@ class Experiment:
 
         """
         df_train, df_test = self.df_train, self.df_test
+        if self.split3:
+            df_dev = self.df_dev
+        else:
+            df_dev = None
         feats_name = "_".join(ast.literal_eval(glob_conf.config["DATA"]["databases"]))
         self.feats_test, self.feats_train = pd.DataFrame(), pd.DataFrame()
+        if self.split3:
+            self.feats_dev = pd.DataFrame()
+        else:
+            self.feats_dev = None
         feats_types = self.util.config_val("FEATS", "type", "os")
         # Ensure feats_types is always a list of strings
         if isinstance(feats_types, str):
@@ -348,20 +439,30 @@ class Experiment:
         # for some models no features are needed
         if len(feats_types) == 0:
             self.util.debug("no feature extractor specified.")
-            self.feats_train, self.feats_test = pd.DataFrame(), pd.DataFrame()
             return
-        self.feature_extractor = FeatureExtractor(
-            df_train, feats_types, feats_name, "train"
-        )
-        self.feats_train = self.feature_extractor.extract()
-        self.feature_extractor = FeatureExtractor(
-            df_test, feats_types, feats_name, "test"
-        )
-        self.feats_test = self.feature_extractor.extract()
-        self.util.debug(
-            f"All features: train shape : {self.feats_train.shape}, test"
-            f" shape:{self.feats_test.shape}"
-        )
+        if not self.train_empty:
+            self.feature_extractor = FeatureExtractor(
+                df_train, feats_types, feats_name, "train"
+            )
+            self.feats_train = self.feature_extractor.extract()
+        if not self.test_empty:
+            self.feature_extractor = FeatureExtractor(
+                df_test, feats_types, feats_name, "test"
+            )
+            self.feats_test = self.feature_extractor.extract()
+        if self.split3:
+            if not self.dev_empty:
+                self.feature_extractor = FeatureExtractor(
+                    df_dev, feats_types, feats_name, "dev"
+                )
+                self.feats_dev = self.feature_extractor.extract()
+                shps = f"{self.feats_train.shape}/{self.feats_dev.shape}/{self.feats_test.shape}"
+                self.util.debug(f"Train/dev/test features:{shps}")
+        else:
+            self.util.debug(
+                f"All features: train shape : {self.feats_train.shape}, test"
+                f" shape:{self.feats_test.shape}"
+            )
         if self.feats_train.shape[0] < self.df_train.shape[0]:
             self.util.warn(
                 f"train feats ({self.feats_train.shape[0]}) != train labels"
@@ -378,14 +479,16 @@ class Experiment:
             )
             self.df_test = self.df_test[self.df_test.index.isin(self.feats_test.index)]
             self.util.warn(f"new test labels shape: {self.df_test.shape[0]}")
+        if self.split3:
+            if self.feats_dev.shape[0] < self.df_dev.shape[0]:
+                self.util.warn(
+                    f"dev feats ({self.feats_dev.shape[0]}) != dev labels"
+                    f" ({self.df_dev.shape[0]})"
+                )
+                self.df_dev = self.df_dev[self.df_dev.index.isin(self.feats_dev.index)]
+                self.util.warn(f"new dev labels shape: {self.df_dev.shape[0]}")
 
         self._check_scale()
-        # store = self.util.get_path("store")
-        # store_format = self.util.config_val("FEATS", "store_format", "pkl")
-        # storage = f"{store}test_feats.{store_format}"
-        # self.util.write_store(self.feats_test, storage, store_format)
-        # storage = f"{store}train_feats.{store_format}"
-        # self.util.write_store(self.feats_train, storage, store_format)
 
     def augment(self):
         """Augment the selected samples."""
@@ -409,10 +512,8 @@ class Experiment:
         return df_ret
 
     def autopredict(self):
-        """
-        Predict labels for samples with existing models and add to the dataframe.
-        """
-        sample_selection = self.util.config_val("PREDICT", "split", "all")
+        """Predict labels for samples with existing models and add to the dataframe."""
+        sample_selection = self.util.config_val("PREDICT", "sample_selection", "all")
         if sample_selection == "all":
             df = pd.concat([self.df_train, self.df_test])
         elif sample_selection == "train":
@@ -424,9 +525,16 @@ class Experiment:
                 f"unknown augmentation selection specifier {sample_selection},"
                 " should be [all | train | test]"
             )
-        targets = self.util.config_val_list("PREDICT", "targets", ["gender"])
+        targets = self.util.config_val_list("PREDICT", "targets", None)
+        if targets is None:
+            self.util.error("no prediction target specified")
         for target in targets:
-            if target == "gender":
+            if target == "speaker":
+                from nkululeko.autopredict.ap_sid import SIDPredictor
+
+                predictor = SIDPredictor(df)
+                df = predictor.predict(sample_selection)
+            elif target == "gender":
                 from nkululeko.autopredict.ap_gender import GenderPredictor
 
                 predictor = GenderPredictor(df)
@@ -461,6 +569,11 @@ class Experiment:
 
                 predictor = STOIPredictor(df)
                 df = predictor.predict(sample_selection)
+            elif target == "text":
+                from nkululeko.autopredict.ap_text import TextPredictor
+
+                predictor = TextPredictor(df, self.util)
+                df = predictor.predict(sample_selection)
             elif target == "arousal":
                 from nkululeko.autopredict.ap_arousal import ArousalPredictor
 
@@ -476,6 +589,11 @@ class Experiment:
 
                 predictor = DominancePredictor(df)
                 df = predictor.predict(sample_selection)
+            elif target == "emotion":
+                from nkululeko.autopredict.ap_emotion import EmotionPredictor
+
+                predictor = EmotionPredictor(df)
+                df = predictor.predict(sample_selection)
             else:
                 self.util.error(f"unknown auto predict target: {target}")
         return df
@@ -560,11 +678,27 @@ class Experiment:
 
         # check if a scatterplot should be done
         scatter_var = eval(self.util.config_val("EXPL", "scatter", "False"))
+
+        # Priority: use [EXPL][scatter.target] if available, otherwise use [DATA][target] value
+        if hasattr(self, "target") and self.target != "none":
+            default_scatter_target = f"['{self.target}']"
+        else:
+            default_scatter_target = "['class_label']"
+
         scatter_target = self.util.config_val(
-            "EXPL", "scatter.target", "['class_label']"
+            "EXPL", "scatter.target", default_scatter_target
         )
+
+        if scatter_target == default_scatter_target:
+            self.util.debug(
+                f"scatter.target using default from [DATA][target]: {scatter_target}"
+            )
+        else:
+            self.util.debug(
+                f"scatter.target from [EXPL][scatter.target]: {scatter_target}"
+            )
         if scatter_var:
-            scatters = ast.literal_eval(glob_conf.config["EXPL"]["scatter"])
+            scatters = ast.literal_eval(scatter_target)
             scat_targets = ast.literal_eval(scatter_target)
             plots = Plots()
             for scat_target in scat_targets:
@@ -584,9 +718,35 @@ class Experiment:
                             df_feats, df_labels, f"{scat_target}_bins", scatter
                         )
 
+        # check if t-SNE plot should be generated
+        tsne = eval(self.util.config_val("EXPL", "tsne", "False"))
+        if tsne:
+            target_column = self.util.config_val("DATA", "target", "emotion")
+            plots = Plots()
+            self.util.debug("generating t-SNE plot...")
+            plots.scatter_plot(df_feats, df_labels, target_column, "tsne")
+
+        # check if UMAP plot should be generated
+        umap_plot = eval(self.util.config_val("EXPL", "umap", "False"))
+        if umap_plot:
+            target_column = self.util.config_val("DATA", "target", "emotion")
+            plots = Plots()
+            self.util.debug("generating UMAP plot...")
+            plots.scatter_plot(df_feats, df_labels, target_column, "umap")
+
+        # check if PCA plot should be generated
+        pca_plot = eval(self.util.config_val("EXPL", "pca", "False"))
+        if pca_plot:
+            target_column = self.util.config_val("DATA", "target", "emotion")
+            plots = Plots()
+            self.util.debug("generating PCA plot...")
+            plots.scatter_plot(df_feats, df_labels, target_column, "pca")
+
     def _check_scale(self):
         self.util.save_to_store(self.feats_train, "feats_train")
         self.util.save_to_store(self.feats_test, "feats_test")
+        if self.split3:
+            self.util.save_to_store(self.feats_dev, "feats_dev")
         scale_feats = self.util.config_val("FEATS", "scale", False)
         # print the scale
         self.util.debug(f"scaler: {scale_feats}")
@@ -597,6 +757,8 @@ class Experiment:
                 self.feats_train,
                 self.feats_test,
                 scale_feats,
+                dev_x=self.df_dev,
+                dev_y=self.feats_dev,
             )
             self.feats_train, self.feats_test = self.scaler_feats.scale()
             # store versions
@@ -605,9 +767,19 @@ class Experiment:
 
     def init_runmanager(self):
         """Initialize the manager object for the runs."""
-        self.runmgr = Runmanager(
-            self.df_train, self.df_test, self.feats_train, self.feats_test
-        )
+        if self.split3:
+            self.runmgr = Runmanager(
+                self.df_train,
+                self.df_test,
+                self.feats_train,
+                self.feats_test,
+                dev_x=self.df_dev,
+                dev_y=self.feats_dev,
+            )
+        else:
+            self.runmgr = Runmanager(
+                self.df_train, self.df_test, self.feats_train, self.feats_test
+            )
 
     def run(self):
         """Do the runs."""
@@ -738,7 +910,6 @@ class Experiment:
         if model.is_ann():
             print("converting to onnx from torch")
         else:
-
             print("converting to onnx from sklearn")
         # save the rest
         f = open(filename, "wb")
