@@ -1,18 +1,21 @@
 # feats_analyser.py
 import ast
+import os
+
+import matplotlib.pyplot as plt
 import pandas as pd
 from sklearn.inspection import permutation_importance
+from sklearn.linear_model import LinearRegression
 from sklearn.linear_model import LogisticRegression
 from sklearn.tree import DecisionTreeClassifier
-from sklearn.linear_model import LinearRegression
 from sklearn.tree import DecisionTreeRegressor
-import matplotlib.pyplot as plt
-from nkululeko.utils.util import Util
-from nkululeko.utils.stats import normalize
-from nkululeko.plots import Plots
+
 import nkululeko.glob_conf as glob_conf
-from nkululeko.reporting.report_item import ReportItem
+from nkululeko.plots import Plots
 from nkululeko.reporting.defines import Header
+from nkululeko.reporting.report_item import ReportItem
+from nkululeko.utils.stats import normalize
+from nkululeko.utils.util import Util
 
 
 class FeatureAnalyser:
@@ -50,28 +53,62 @@ class FeatureAnalyser:
 
         name = "my_shap_values"
         if not self.util.exist_pickle(name):
+            # get model name
+            model_name = self.util.get_model_type()
+            if hasattr(model, "predict_shap"):
+                model_func = model.predict_shap
+            elif hasattr(model, "clf"):
+                model_func = model.clf.predict
+            else:
+                raise Exception("Model not supported for SHAP analysis")
+
+            self.util.debug(f"using SHAP explainer for {model_name} model")
 
             explainer = shap.Explainer(
-                model.predict_shap,
+                model_func,
                 self.features,
                 output_names=glob_conf.labels,
                 algorithm="permutation",
                 npermutations=5,
             )
+
             self.util.debug("computing SHAP values...")
             shap_values = explainer(self.features)
             self.util.to_pickle(shap_values, name)
         else:
             shap_values = self.util.from_pickle(name)
-        plt.tight_layout()
-        shap.plots.bar(shap_values)
-        fig_dir = self.util.get_path("fig_dir") + "../"  # one up because of the runs
-        exp_name = self.util.get_exp_name(only_data=True)
+        # Create SHAP summary plot instead
+        fig, ax = plt.subplots(figsize=(10, 6))
+        shap.plots.bar(shap_values, ax=ax, show=False)
+        fig_dir = os.path.join(self.util.get_path("fig_dir"), "..")
+
         format = self.util.config_val("PLOT", "format", "png")
-        filename = f"_SHAP_{model.name}"
-        filename = f"{fig_dir}{exp_name}{filename}.{format}"
-        plt.savefig(filename)
-        self.util.debug(f"plotted SHAP feature importance tp {filename}")
+        feat_type = self.util.get_feattype_name()
+        filename = f"SHAP_{feat_type}_{model.name}.{format}"
+        filename = os.path.join(fig_dir, filename)
+
+        fig.savefig(filename, dpi=300, bbox_inches="tight")
+        plt.close(fig)
+
+        # print and save SHAP feature importance
+        max_feat_num = len(self.features.columns)
+        shap_importance_values = shap_values.abs.mean(0).values
+
+        feature_cols = self.features.columns
+        feature_importance = pd.DataFrame(
+            shap_importance_values[:max_feat_num],
+            index=feature_cols,
+            columns=["importance"],
+        ).sort_values("importance", ascending=False)
+
+        self.util.debug(
+            f"SHAP analysis, features = {feature_importance.index.tolist()}"
+        )
+        # Save to CSV (save all features, not just top ones)
+        csv_filename = os.path.join(fig_dir, f"SHAP_{feat_type}_importance_{model.name}.csv")
+        feature_importance.to_csv(csv_filename)
+        self.util.debug(f"Saved SHAP feature importance to {csv_filename}")
+        self.util.debug(f"plotted SHAP feature importance to {filename}")
 
     def analyse(self):
         models = ast.literal_eval(self.util.config_val("EXPL", "model", "['log_reg']"))
@@ -104,6 +141,12 @@ class FeatureAnalyser:
                     covariance_type = self.util.config_val(
                         "MODEL", "GMM_covariance_type", "full"
                     )
+                    allowed_cov_types = ["full", "tied", "diag", "spherical"]
+                    if covariance_type not in allowed_cov_types:
+                        self.util.error(
+                            f"Invalid covariance_type '{covariance_type}', must be one of {allowed_cov_types}. Using default 'full'."
+                        )
+                        covariance_type = "full"
                     model = mixture.GaussianMixture(
                         n_components=n_components, covariance_type=covariance_type
                     )
@@ -139,8 +182,8 @@ class FeatureAnalyser:
                 elif model_s == "svm":
                     from sklearn.svm import SVC
 
-                    c = float(self.util.config_val("MODEL", "C_val", "0.001"))
-                    model = SVC(kernel="linear", C=c, gamma="scale")
+                    c = float(self.util.config_val("MODEL", "C_val", "1.0"))
+                    model = SVC(kernel="linear", C=c, gamma="scale", random_state=42)
                     result_importances[model_s] = self._get_importance(
                         model, permutation
                     )
@@ -149,7 +192,7 @@ class FeatureAnalyser:
                         plots = Plots()
                         plots.plot_tree(model, self.features)
                 elif model_s == "tree":
-                    model = DecisionTreeClassifier()
+                    model = DecisionTreeClassifier(random_state=42)
                     result_importances[model_s] = self._get_importance(
                         model, permutation
                     )
@@ -160,7 +203,9 @@ class FeatureAnalyser:
                 elif model_s == "xgb":
                     from xgboost import XGBClassifier
 
-                    model = XGBClassifier(enable_categorical=True, tree_method="hist")
+                    model = XGBClassifier(
+                        enable_categorical=True, tree_method="hist", random_state=42
+                    )
                     self.labels = self.labels.astype("category")
                     result_importances[model_s] = self._get_importance(
                         model, permutation
@@ -205,7 +250,7 @@ class FeatureAnalyser:
                         model, permutation
                     )
                 elif model_s == "xgr":
-                    from xgboost import XGBClassifier
+                    from xgboost import XGBRegressor
 
                     model = XGBRegressor()
                     result_importances[model_s] = self._get_importance(
@@ -247,18 +292,17 @@ class FeatureAnalyser:
             title += "\n based on feature permutation"
         ax.set(title=title)
         plt.tight_layout()
-        fig_dir = self.util.get_path("fig_dir") + "../"  # one up because of the runs
-        exp_name = self.util.get_exp_name(only_data=True)
+        fig_dir = self.util.get_path("fig_dir")
         format = self.util.config_val("PLOT", "format", "png")
-        filename = f"_EXPL_{model_name}"
+        filename = f"EXPL_{model_name}"
         if permutation:
             filename += "_perm"
-        filename = f"{fig_dir}{exp_name}{filename}.{format}"
+        filename = f"{fig_dir}{filename}.{format}"
         plt.savefig(filename)
         fig = ax.figure
         fig.clear()
         plt.close(fig)
-        caption = f"Feature importance"
+        caption = "Feature importance"
         if permutation:
             caption += " based on permutation of features."
         glob_conf.report.add_item(
@@ -270,12 +314,16 @@ class FeatureAnalyser:
             )
         )
 
+        # print feature importance values to file and debug and save to result
+        self.util.debug(
+            f"Importance features from {model_name}: features = \n{df_imp['feats'].values.tolist()}"
+        )
         # result file
         res_dir = self.util.get_path("res_dir")
         filename = f"_EXPL_{model_name}"
         if permutation:
             filename += "_perm"
-        filename = f"{res_dir}{self.util.get_exp_name(only_data=True)}{filename}_{model_name}.txt"
+        filename = f"{res_dir}{self.util.get_exp_name(only_data=True)}{filename}_{max_feat_num}_fi.txt"
         with open(filename, "w") as text_file:
             text_file.write(
                 "features in order of decreasing importance according to model"
@@ -283,6 +331,7 @@ class FeatureAnalyser:
             )
 
         df_imp.to_csv(filename, mode="a")
+        self.util.debug(f"Saved feature importance values to {filename}")
 
         # check if feature distributions should be plotted
         plot_feats = self.util.config_val("EXPL", "feature_distributions", False)
