@@ -4,10 +4,10 @@ import os
 import audeer
 import audinterface
 import audonnx
-import nkululeko.glob_conf as glob_conf
 import numpy as np
-import pandas as pd
 import torch
+
+import nkululeko.glob_conf as glob_conf
 from nkululeko.feat_extract.featureset import Featureset
 
 
@@ -30,8 +30,7 @@ class AudmodelSet(Featureset):
         if not os.path.isdir(model_root):
             cache_root = audeer.mkdir("cache")
             model_root = audeer.mkdir(model_root)
-            archive_path = audeer.download_url(
-                model_url, cache_root, verbose=True)
+            archive_path = audeer.download_url(model_url, cache_root, verbose=True)
             audeer.extract_archive(archive_path, model_root)
         cuda = "cuda" if torch.cuda.is_available() else "cpu"
         device = self.util.config_val("MODEL", "device", cuda)
@@ -61,7 +60,7 @@ class AudmodelSet(Featureset):
                 },
                 sampling_rate=16000,
                 resample=True,
-                num_workers=5,
+                num_workers=self.n_jobs,
                 verbose=True,
             )
             self.df = hidden_states.process_index(self.data_df.index)
