@@ -1,7 +1,8 @@
-""" feats_snr.py is to estimate snr.
+"""feats_snr.py is to estimate snr.
 
 SNR (signal to noise ratio) is extracted as acoustic features.
 """
+
 import os
 
 import audiofile
@@ -11,7 +12,6 @@ from tqdm import tqdm
 import nkululeko.glob_conf as glob_conf
 from nkululeko.autopredict.estimate_snr import SNREstimator
 from nkululeko.feat_extract.featureset import Featureset
-from nkululeko.utils.util import Util
 
 
 class SNRSet(Featureset):
@@ -28,8 +28,7 @@ class SNRSet(Featureset):
         store = self.util.get_path("store")
         store_format = self.util.config_val("FEATS", "store_format", "pkl")
         storage = f"{store}{self.name}.{store_format}"
-        extract = self.util.config_val(
-            "FEATS", "needs_feature_extraction", False)
+        extract = self.util.config_val("FEATS", "needs_feature_extraction", False)
         no_reuse = eval(self.util.config_val("FEATS", "no_reuse", "False"))
         if extract or no_reuse or not os.path.isfile(storage):
             self.util.debug("estimating SNR, this might take a while...")
@@ -46,8 +45,7 @@ class SNRSet(Featureset):
                 snr = self.get_snr(signal[0], sampling_rate)
                 snr_series[idx] = snr
             print("")
-            self.df = pd.DataFrame(
-                snr_series.values.tolist(), index=self.data_df.index)
+            self.df = pd.DataFrame(snr_series.values.tolist(), index=self.data_df.index)
             self.df.columns = ["snr"]
             self.util.write_store(self.df, storage, store_format)
             try:
