@@ -1,4 +1,4 @@
-""" feats_wav2vec2.py
+"""feats_wav2vec2.py.
 feat_types example = [wav2vec2-large-robust-ft-swbd-300h,
 wav2vec2-xls-r-2b, wav2vec2-large, wav2vec2-large-xlsr-53, wav2vec2-base]
 
@@ -7,12 +7,15 @@ Currently only supports wav2vec2
 """
 
 import os
-from tqdm import tqdm
+
 import pandas as pd
 import torch
 import torchaudio
-from transformers import Wav2Vec2FeatureExtractor, Wav2Vec2Model
+from tqdm import tqdm
 import transformers
+from transformers import Wav2Vec2FeatureExtractor
+from transformers import Wav2Vec2Model
+
 from nkululeko.feat_extract.featureset import Featureset
 import nkululeko.glob_conf as glob_conf
 
@@ -46,8 +49,11 @@ class Wav2vec2(Featureset):
         hidden_layer = int(self.util.config_val("FEATS", "wav2vec2.layer", "0"))
         config.num_hidden_layers = layer_num - hidden_layer
         self.util.debug(f"using hidden layer #{config.num_hidden_layers}")
+
         self.processor = Wav2Vec2FeatureExtractor.from_pretrained(model_path)
-        self.model = Wav2Vec2Model.from_pretrained(model_path, config=config).to(self.device)
+        self.model = Wav2Vec2Model.from_pretrained(model_path, config=config).to(
+            self.device
+        )
         print(f"intialized Wav2vec model on {self.device}")
         self.model.eval()
         self.model_initialized = True
