@@ -4,6 +4,7 @@ import pandas as pd
 
 from nkululeko import glob_conf
 from nkululeko.utils.util import Util
+from nkululeko.balance import DataBalancer
 
 
 class Modelrunner:
@@ -53,8 +54,6 @@ class Modelrunner:
             # epochs are handled by Huggingface API
             self.model.train()
             report = self.model.predict()
-            # todo: findout the best epoch, no need
-            # since oad_best_model_at_end is given in training args
             epoch = epoch_num
             report.set_id(self.run, epoch)
             plot_name = self.util.get_plot_name() + f"_{self.run}_{epoch:03d}_cnf"
@@ -68,7 +67,7 @@ class Modelrunner:
                 report.plot_confmatrix(plot_name, epoch)
         else:
             # for all epochs
-            for epoch in range(epoch_num):
+            for epoch_index, epoch in enumerate(range(epoch_num)):
                 if only_test:
                     self.model.load(self.run, epoch)
                     self.util.debug(f"reusing model: {self.model.store_path}")
@@ -118,18 +117,34 @@ class Modelrunner:
                             f"reached patience ({str(patience)}): early stopping"
                         )
                         break
-        # After training, report the best performance and epoch
-        best_report = reports[self.best_epoch]
-        # self.util.debug(f"Best score at epoch: {self.best_epoch}, UAR: {self.best_performance}") # move to reporter below
-
-        if not plot_epochs:
-            # Do at least one confusion matrix plot
-            self.util.debug(f"plotting confusion matrix to {plot_name}")
-            best_report.plot_confmatrix(plot_name, self.best_epoch)
         return reports, epoch
 
+    def eval_last_model(self, df_test, feats_test):
+        self.model.reset_test(df_test, feats_test)
+        report = self.model.predict()
+        report.set_id(self.run, 0)
+        return report
+
+    def eval_specific_model(self, model, df_test, feats_test):
+        self.model = model
+        self.util.debug(f"evaluating model: {self.model.store_path}")
+        self.model.reset_test(df_test, feats_test)
+        report = self.model.predict()
+        report.set_id(self.run, 0)
+        return report
+
+    def _check_balancing(self):
+        if self.util.config_val("EXP", "balancing", False):
+            self.util.debug("balancing data")
+            self.df_train, self.df_test = self.util.balance_data(
+                self.df_train, self.df_test
+            )
+            self.util.debug(f"new train size: {self.df_train.shape}")
+            self.util.debug(f"new test size: {self.df_test.shape}")
+
     def _select_model(self, model_type):
         self._check_balancing()
+        self._check_feature_balancing()
 
         if model_type == "svm":
             from nkululeko.models.model_svm import SVM_model
@@ -230,45 +245,24 @@ class Modelrunner:
             )
         return self.model
 
-    def _check_balancing(self):
+    def _check_feature_balancing(self):
+        """Check and apply feature balancing using the dedicated DataBalancer class."""
         balancing = self.util.config_val("FEATS", "balancing", False)
         if balancing:
-            orig_size = self.feats_train.shape[0]
-            self.util.debug(f"balancing the training features with: {balancing}")
-            if balancing == "ros":
-                from imblearn.over_sampling import RandomOverSampler
+            self.util.debug("Applying feature balancing using DataBalancer")
 
-                sampler = RandomOverSampler(random_state=42)
-                X_res, y_res = sampler.fit_resample(
-                    self.feats_train, self.df_train[self.target]
-                )
-            elif balancing == "smote":
-                from imblearn.over_sampling import SMOTE
+            # Get random state from config, fallback to 42 for backward compatibility
+            random_state = int(
+                self.util.config_val("FEATS", "balancing_random_state", 42)
+            )
 
-                sampler = SMOTE(random_state=42)
-                X_res, y_res = sampler.fit_resample(
-                    self.feats_train, self.df_train[self.target]
-                )
-            elif balancing == "adasyn":
-                from imblearn.over_sampling import ADASYN
+            # Initialize the data balancer with configurable random state
+            balancer = DataBalancer(random_state=random_state)
 
-                sampler = ADASYN(random_state=42)
-                X_res, y_res = sampler.fit_resample(
-                    self.feats_train, self.df_train[self.target]
-                )
-            else:
-                self.util.error(
-                    f"unknown balancing algorithm: {balancing} (should be [ros|smote|adasyn])"
-                )
-
-            self.feats_train = X_res
-            self.df_train = pd.DataFrame({self.target: y_res}, index=X_res.index)
-            self.util.debug(
-                f"balanced with: {balancing}, new size: {X_res.shape[0]} (was {orig_size})"
+            # Apply balancing
+            self.df_train, self.feats_train = balancer.balance_features(
+                df_train=self.df_train,
+                feats_train=self.feats_train,
+                target_column=self.target,
+                method=balancing,
             )
-            le = glob_conf.label_encoder
-            res = y_res.value_counts()
-            resd = {}
-            for i, e in enumerate(le.inverse_transform(res.index.values)):
-                resd[e] = res.values[i]
-            self.util.debug(f"{resd})")
