@@ -1,4 +1,4 @@
-""" 
+"""
 model_cnn.py
 
 Inspired by code from Su Lei
@@ -6,23 +6,21 @@ Inspired by code from Su Lei
 """
 
 import ast
-from collections import OrderedDict
 
 import numpy as np
 import pandas as pd
-from PIL import Image
-from sklearn.metrics import recall_score
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from torch.utils.data import Dataset
 import torchvision.transforms as transforms
+from PIL import Image
+from sklearn.metrics import recall_score
+from torch.utils.data import Dataset
 
 import nkululeko.glob_conf as glob_conf
 from nkululeko.losses.loss_softf1loss import SoftF1Loss
 from nkululeko.models.model import Model
 from nkululeko.reporting.reporter import Reporter
-from nkululeko.utils.util import Util
 
 
 class CNNModel(Model):
@@ -55,7 +53,7 @@ class CNNModel(Model):
             )
         else:
             self.util.error(f"unknown loss function: {criterion}")
-        self.util.debug(f"using model with cross entropy loss function")
+        self.util.debug("using model with cross entropy loss function")
         # set up the model
         # cuda = "cuda" if torch.cuda.is_available() else "cpu"
         self.device = self.util.config_val("MODEL", "device", "cpu")
@@ -80,7 +78,7 @@ class CNNModel(Model):
         # batch size
         self.batch_size = int(self.util.config_val("MODEL", "batch_size", 8))
         # number of parallel processes
-        self.num_workers = int(self.util.config_val("MODEL", "num_workers", 5))
+        self.num_workers = self.n_jobs
 
         # set up the data_loaders
 
@@ -100,13 +98,13 @@ class CNNModel(Model):
             train_set,
             batch_size=self.batch_size,
             shuffle=True,
-            num_workers=self.num_workers,
+            num_workers=self.n_jobs,
         )
         self.testloader = torch.utils.data.DataLoader(
             test_set,
             batch_size=self.batch_size,
             shuffle=False,
-            num_workers=self.num_workers,
+            num_workers=self.n_jobs,
         )
 
     class Dataset_image(Dataset):
@@ -136,7 +134,7 @@ class CNNModel(Model):
             test_set,
             batch_size=self.batch_size,
             shuffle=False,
-            num_workers=self.num_workers,
+            num_workers=self.n_jobs,
         )
 
     def reset_test(self, df_test, feats_test):
