@@ -11,7 +11,6 @@ import nkululeko.glob_conf as glob_conf
 from nkululeko.losses.loss_softf1loss import SoftF1Loss
 from nkululeko.models.model import Model
 from nkululeko.reporting.reporter import Reporter
-from nkululeko.utils.util import Util
 
 
 class MLPModel(Model):
@@ -32,6 +31,10 @@ class MLPModel(Model):
         super().set_model_type("ann")
         self.name = "mlp"
         self.target = glob_conf.config["DATA"]["target"]
+        manual_seed = eval(self.util.config_val("MODEL", "manual_seed", "True"))
+        if manual_seed:
+            self.util.debug(f"seeding random to {23}")
+            torch.manual_seed(23)
         labels = glob_conf.labels
         self.class_num = len(labels)
         # set up loss criterion
@@ -44,7 +47,7 @@ class MLPModel(Model):
             )
         else:
             self.util.error(f"unknown loss function: {criterion}")
-        self.util.debug(f"using model with cross entropy loss function")
+        self.util.debug("using model with cross entropy loss function")
         # set up the model, use GPU if availabe
         cuda = "cuda" if torch.cuda.is_available() else "cpu"
         self.device = self.util.config_val("MODEL", "device", cuda)
@@ -71,7 +74,7 @@ class MLPModel(Model):
         # batch size
         self.batch_size = int(self.util.config_val("MODEL", "batch_size", 8))
         # number of parallel processes
-        self.num_workers = int(self.util.config_val("MODEL", "num_workers", 5))
+        self.num_workers = self.n_jobs
         if feats_train.isna().to_numpy().any():
             self.util.debug(
                 f"Model, train: replacing {feats_train.isna().sum().sum()} NANs"
@@ -92,6 +95,8 @@ class MLPModel(Model):
 
     def reset_test(self, df_test, feats_test):
         self.testloader = self.get_loader(feats_test, df_test, False)
+        self.df_test = df_test
+        self.feats_test = feats_test
 
     def train(self):
         self.model.train()
