@@ -1,15 +1,10 @@
 # model_mlp.py
 import ast
 from collections import OrderedDict
-import os
 
 import numpy as np
 import torch
-
-from audmetric import concordance_cc
-from audmetric import mean_absolute_error
-from audmetric import mean_squared_error
-from traitlets import default
+from audmetric import concordance_cc, mean_absolute_error, mean_squared_error
 
 import nkululeko.glob_conf as glob_conf
 from nkululeko.losses.loss_ccc import ConcordanceCorCoeff
@@ -41,6 +36,10 @@ class MLP_Reg_model(Model):
         else:
             self.util.error(f"unknown loss function: {criterion}")
         self.util.debug(f"training model with {criterion} loss function")
+        manual_seed = eval(self.util.config_val("MODEL", "manual_seed", "True"))
+        if manual_seed:
+            self.util.debug(f"seeding random to {23}")
+            torch.manual_seed(23)
         # set up the model
         cuda = "cuda" if torch.cuda.is_available() else "cpu"
         self.device = self.util.config_val("MODEL", "device", cuda)
@@ -64,7 +63,7 @@ class MLP_Reg_model(Model):
         # batch size
         self.batch_size = int(self.util.config_val("MODEL", "batch_size", 8))
         # number of parallel processes
-        self.num_workers = int(self.util.config_val("MODEL", "num_workers", 5))
+        self.num_workers = self.n_jobs
         # set up the data_loaders
         if feats_train.isna().to_numpy().any():
             self.util.debug(
@@ -117,7 +116,7 @@ class MLP_Reg_model(Model):
             dataset=data_set,
             batch_size=self.batch_size,
             shuffle=shuffle,
-            num_workers=self.num_workers,
+            num_workers=self.n_jobs,
         )
         return loader
 
@@ -175,6 +174,9 @@ class MLP_Reg_model(Model):
             logits = model(features.to(device)).reshape(-1)
             loss = self.criterion(logits, labels.to(device))
             # print(f'loss: {loss.item()}')
+            if torch.isnan(loss):
+                # possible that ccc returns NaN if batch contains only one value
+                continue
             losses.append(loss.item())
             optimizer.zero_grad()
             loss.backward()
