@@ -1,6 +1,7 @@
 # model_tree.py
 
 from sklearn.tree import DecisionTreeClassifier
+
 from nkululeko.models.model import Model
 
 
@@ -12,4 +13,4 @@ class Tree_model(Model):
     def __init__(self, df_train, df_test, feats_train, feats_test):
         super().__init__(df_train, df_test, feats_train, feats_test)
         self.name = "tree"
-        self.clf = DecisionTreeClassifier()  # set up the classifier
+        self.clf = DecisionTreeClassifier(random_state=42)  # set up the classifier
