@@ -1,5 +1,6 @@
 # plots.py
 import ast
+import os
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -8,6 +9,8 @@ from scipy import stats
 import seaborn as sns
 from sklearn.manifold import TSNE
 
+from audmetric import concordance_cc as ccc
+
 import nkululeko.glob_conf as glob_conf
 from nkululeko.reporting.defines import Header
 from nkululeko.reporting.report_item import ReportItem
@@ -21,8 +24,11 @@ class Plots:
         self.util = Util("plots")
         self.format = self.util.config_val("PLOT", "format", "png")
         self.target = self.util.config_val("DATA", "target", "emotion")
+        self.with_ccc = eval(self.util.config_val("PLOT", "ccc", "False"))
+        self.type_s = "samples"
 
     def plot_distributions_speaker(self, df):
+        self.type_s = "speaker"
         df_speakers = pd.DataFrame()
         pd.options.mode.chained_assignment = None  # default='warn'
         for s in df.speaker.unique():
@@ -30,11 +36,9 @@ class Plots:
             df_speaker["samplenum"] = df_speaker.shape[0]
             df_speakers = pd.concat([df_speakers, df_speaker.head(1)])
         # plot the distribution of samples per speaker
-        # one up because of the runs
-        fig_dir = self.util.get_path("fig_dir") + "../"
-        self.util.debug(f"plotting samples per speaker")
+        self.util.debug("plotting samples per speaker")
         if "gender" in df_speakers:
-            filename = f"samples_value_counts"
+            filename = "samples_value_counts"
             ax = (
                 df_speakers.groupby("samplenum")["gender"]
                 .value_counts()
@@ -46,7 +50,7 @@ class Plots:
                     rot=0,
                 )
             )
-            ax.set_ylabel(f"number of speakers")
+            ax.set_ylabel("number of speakers")
             ax.set_xlabel("number of samples")
             self.save_plot(
                 ax,
@@ -58,7 +62,7 @@ class Plots:
 
             # fig.clear()
         else:
-            filename = f"samples_value_counts"
+            filename = "samples_value_counts"
             ax = (
                 df_speakers["samplenum"]
                 .value_counts()
@@ -84,9 +88,10 @@ class Plots:
 
     def plot_distributions(self, df, type_s="samples"):
         class_label, df = self._check_binning("class_label", df)
-        attributes = ast.literal_eval(
-            self.util.config_val("EXPL", "value_counts", False)
-        )
+        value_counts_conf = self.util.config_val("EXPL", "value_counts", False)
+        if not isinstance(value_counts_conf, str):
+            value_counts_conf = str(value_counts_conf)
+        attributes = ast.literal_eval(value_counts_conf)
         # always plot the distribution of the main attribute
         filename = f"{class_label}_distribution"
         if self.util.is_categorical(df[class_label]):
@@ -213,11 +218,11 @@ class Plots:
 
     def save_plot(self, ax, caption, header, filename, type_s):
         # one up because of the runs
-        fig_dir = self.util.get_path("fig_dir") + "../"
+        fig_dir = os.path.dirname(self.util.get_path("fig_dir"))
         fig_plots = ax.figure
         # avoid warning
         # plt.tight_layout()
-        img_path = f"{fig_dir}{filename}_{type_s}.{self.format}"
+        img_path = os.path.join(fig_dir, f"{filename}_{type_s}.{self.format}")
         plt.savefig(img_path)
         plt.close(fig_plots)
         self.util.debug(f"Saved plot to {img_path}")
@@ -241,53 +246,111 @@ class Plots:
 
     def _plot2cont_cat(self, df, cont1, cont2, cat, ylab):
         """Plot relation of two continuous distributions with one categorical."""
-        pearson = stats.pearsonr(df[cont1], df[cont2])
+        plot_df = df[[cont1, cont2, cat]].copy()
+        if cont2 == "class_label":
+            plot_df = plot_df.rename(columns={cont2: self.target})
+            cont2 = self.target
+        if cont1 == "class_label":
+            plot_df = plot_df.rename(columns={cont1: self.target})
+            cont1 = self.target
+        if cat == "class_label":
+            plot_df = plot_df.rename(columns={cat: self.target})
+            cat = self.target
+        pearson = stats.pearsonr(plot_df[cont1], plot_df[cont2])
         # trunc to three digits
         pearson = int(pearson[0] * 1000) / 1000
         pearson_string = f"PCC: {pearson}"
-        ax = sns.lmplot(data=df, x=cont1, y=cont2, hue=cat)
-        caption = f"{ylab} {df.shape[0]}. {pearson_string}"
+        ccc_string = ""
+        if self.with_ccc:
+            ccc_val = ccc(plot_df[cont1], plot_df[cont2])
+            ccc_val = int(ccc_val * 1000) / 1000
+            ccc_string = f"CCC: {ccc_val}"
+        ax = sns.lmplot(data=plot_df, x=cont1, y=cont2, hue=cat)
+        caption = f"{ylab} {plot_df.shape[0]}. {pearson_string} {ccc_string}"
         ax.figure.suptitle(caption)
         return ax, caption
 
     def _plot2cont(self, df, col1, col2, ylab):
         """Plot relation of two continuous distributions."""
-        pearson = stats.pearsonr(df[col1], df[col2])
+        plot_df = df[[col1, col2]].copy()
+        # rename "class_label" to the original target
+        if col2 == "class_label":
+            plot_df = plot_df.rename(columns={col2: self.target})
+            col2 = self.target
+        if col1 == "class_label":
+            plot_df = plot_df.rename(columns={col1: self.target})
+            col1 = self.target
+        pearson = stats.pearsonr(plot_df[col1], plot_df[col2])
         # trunc to three digits
         pearson = int(pearson[0] * 1000) / 1000
         pearson_string = f"PCC: {pearson}"
-        ax = sns.lmplot(data=df, x=col1, y=col2)
-        caption = f"{ylab} {df.shape[0]}. {pearson_string}"
+        ccc_string = ""
+        if self.with_ccc:
+            ccc_val = ccc(plot_df[col1], plot_df[col2])
+            ccc_val = int(ccc_val * 1000) / 1000
+            ccc_string = f"CCC: {ccc_val}"
+        ax = sns.lmplot(data=plot_df, x=col1, y=col2)
+        caption = f"{ylab} {plot_df.shape[0]}. {pearson_string} {ccc_string}"
         ax.figure.suptitle(caption)
         return ax, caption
 
     def plotcatcont(self, df, cat_col, cont_col, xlab, ylab):
         """Plot relation of categorical distribution with continuous."""
-        dist_type = self.util.config_val("EXPL", "dist_type", "hist")
-        cats, cat_str, es = su.get_effect_size(df, cat_col, cont_col)
-        if dist_type == "hist":
-            ax = sns.histplot(df, x=cont_col, hue=cat_col, kde=True)
-            caption = f"{ylab} {df.shape[0]}. {cat_str} ({cats}):" f" {es}"
+        # rename "class_label" to the original target
+        plot_df = df[[cat_col, cont_col]].copy()
+        if cat_col == "class_label":
+            plot_df = plot_df.rename(columns={cat_col: self.target})
+            cat_col = self.target
+        elif cont_col == "class_label":
+            plot_df = plot_df.rename(columns={cont_col: self.target})
+            cont_col = self.target
+        dist_type = self.util.config_val("EXPL", "dist_type", "kde")
+        fill_areas = eval(self.util.config_val("PLOT", "fill_areas", "False"))
+        max_cat, cat_str, effect_results = su.get_effect_size(
+            plot_df, cat_col, cont_col
+        )
+        self.util.debug(effect_results)
+        self.util.print_results_to_store(
+            f"cohens-d_{self.type_s}", str(effect_results) + "\n"
+        )
+        es = effect_results[max_cat]
+        model_type = self.util.get_model_type()
+        if dist_type == "hist" and model_type != "tree":
+            ax = sns.histplot(plot_df, x=cont_col, hue=cat_col, kde=True)
+            caption = f"{ylab} {plot_df.shape[0]}. {cat_str} ({max_cat}):" f" {es}"
             ax.set_title(caption)
             ax.set_xlabel(f"{cont_col}")
             ax.set_ylabel(f"number of {ylab}")
         else:
             ax = sns.displot(
-                df, x=cont_col, hue=cat_col, kind="kde", fill=True, warn_singular=False
+                plot_df,
+                x=cont_col,
+                hue=cat_col,
+                kind="kde",
+                fill=fill_areas,
+                warn_singular=False,
             )
             ax.set(xlabel=f"{cont_col}")
-            caption = f"{ylab} {df.shape[0]}. {cat_str} ({cats}):" f" {es}"
+            caption = f"{ylab} {plot_df.shape[0]}. {cat_str} ({max_cat}):" f" {es}"
             ax.figure.suptitle(caption)
         return ax, caption
 
     def _plot2cat(self, df, col1, col2, xlab, ylab):
         """Plot relation of 2 categorical distributions."""
-        crosstab = pd.crosstab(index=df[col1], columns=df[col2])
+        plot_df = df[[col1, col2]].copy()
+        # rename "class_label" to the original target
+        if col2 == "class_label":
+            plot_df = plot_df.rename(columns={col2: self.target})
+            col2 = self.target
+        elif col1 == "class_label":
+            plot_df = plot_df.rename(columns={col1: self.target})
+            col1 = self.target
+        crosstab = pd.crosstab(index=plot_df[col1], columns=plot_df[col2])
         res_pval = stats.chi2_contingency(crosstab)
         res_pval = int(res_pval[1] * 1000) / 1000
-        caption = f"{ylab} {df.shape[0]}. P-val chi2: {res_pval}"
+        caption = f"{ylab} {plot_df.shape[0]}. P-val chi2: {res_pval}"
         ax = (
-            df.groupby(col1, observed=False)[col2]
+            plot_df.groupby(col1, observed=False)[col2]
             .value_counts()
             .unstack()
             .plot(kind="bar", stacked=True, title=caption, rot=0)
@@ -298,12 +361,15 @@ class Plots:
 
     def plot_durations(self, df, filename, sample_selection, caption=""):
         # one up because of the runs
-        fig_dir = self.util.get_path("fig_dir") + "../"
+        fig_dir = os.path.join(self.util.get_path("fig_dir"), "..")
         try:
             ax = sns.histplot(df, x="duration", hue="class_label", kde=True)
         except AttributeError as ae:
             self.util.warn(ae)
             ax = sns.histplot(df, x="duration", kde=True)
+        except ValueError as error:
+            self.util.warn(error)
+            ax = sns.histplot(df, x="duration", kde=True)
         min = self.util.to_3_digits(df.duration.min())
         max = self.util.to_3_digits(df.duration.max())
         title = f"Duration distr. for {sample_selection} {df.shape[0]}. min={min}, max={max}"
@@ -312,9 +378,34 @@ class Plots:
         ax.set_ylabel("number of samples")
         fig = ax.figure
         # plt.tight_layout()
-        img_path = f"{fig_dir}{filename}_{sample_selection}.{self.format}"
+        img_path = os.path.join(fig_dir, f"{filename}_{sample_selection}.{self.format}")
         plt.savefig(img_path)
         plt.close(fig)
+        self.util.debug(f"plotted durations to {img_path}")
+        glob_conf.report.add_item(
+            ReportItem(
+                Header.HEADER_EXPLORE,
+                caption,
+                title,
+                img_path,
+            )
+        )
+
+    def plot_speakers(self, df, sample_selection):
+        filename = "speakers"
+        caption = "speakers"
+        # one up because of the runs
+        fig_dir = os.path.join(self.util.get_path("fig_dir"), "..")
+        sns.set_style("whitegrid")  # Set style for chart
+        ax = df["speaker"].value_counts().plot(kind="pie", autopct="%1.1f%%")
+        title = f"Speaker distr. for {sample_selection} {df.shape[0]}."
+        ax.set_title(title)
+        fig = ax.figure
+        # plt.tight_layout()
+        img_path = os.path.join(fig_dir, f"{filename}_{sample_selection}.{self.format}")
+        plt.savefig(img_path)
+        plt.close(fig)
+        self.util.debug(f"plotted speakers to {img_path}")
         glob_conf.report.add_item(
             ReportItem(
                 Header.HEADER_EXPLORE,
@@ -326,7 +417,7 @@ class Plots:
 
     def describe_df(self, name, df, target, filename):
         """Make a stacked barplot of samples and speakers per sex and target values. speaker, gender and target columns must be present"""
-        fig_dir = self.util.get_path("fig_dir") + "../"  # one up because of the runs
+        fig_dir = self.util.get_path("fig_dir")  # + "../"  # one up because of the runs
         sampl_num = df.shape[0]
         sex_col = "gender"
         if target == "gender":
@@ -358,7 +449,7 @@ class Plots:
                     kind="bar", ax=axes, title=f"samples ({sampl_num})"
                 )
             # plt.tight_layout()
-            img_path = f"{fig_dir}{filename}.{self.format}"
+            img_path = os.path.join(fig_dir, f"{filename}.{self.format}")
             plt.savefig(img_path)
             fig.clear()
             plt.close(fig)
@@ -373,11 +464,12 @@ class Plots:
 
     def scatter_plot(self, feats, label_df, label, dimred_type):
         dim_num = int(self.util.config_val("EXPL", "scatter.dim", 2))
-        # one up because of the runs
-        fig_dir = self.util.get_path("fig_dir") + "../"
+        # one up because of the runs (for explore module)
+        fig_dir = os.path.join(self.util.get_path("fig_dir"), "..")
         sample_selection = self.util.config_val("EXPL", "sample_selection", "all")
-        filename = f"{label}_{self.util.get_feattype_name()}_{sample_selection}_{dimred_type}_{str(dim_num)}d"
-        filename = f"{fig_dir}{filename}.{self.format}"
+        exp_name = self.util.get_name()
+        filename = f"{label}_{exp_name}_{self.util.get_feattype_name()}_{sample_selection}_{dimred_type}_{str(dim_num)}d"
+        filename = os.path.join(fig_dir, f"{filename}.{self.format}")
         self.util.debug(f"computing {dimred_type}, this might take a while...")
         data = None
         labels = label_df[label]
@@ -484,12 +576,13 @@ class Plots:
             self.util.error(f"wrong dimension number: {dim_num}")
         fig = ax.figure
         plt.savefig(filename)
+        self.util.debug(f"plotted {dimred_type} scatter plot to {filename}")
         fig.clear()
         plt.close(fig)
         glob_conf.report.add_item(
             ReportItem(
                 Header.HEADER_EXPLORE,
-                f"Scatter plot",
+                "Scatter plot",
                 f"using {dimred_type}",
                 filename,
             )
@@ -510,15 +603,25 @@ class Plots:
         # remove fullstops in the name
         feature_name = feature.replace(".", "-")
         # one up because of the runs
-        fig_dir = self.util.get_path("fig_dir") + "../"
-        filename = f"{fig_dir}feat_dist_{title}_{feature_name}.{self.format}"
+        fig_dir = os.path.join(self.util.get_path("fig_dir"), "..")
+        filename = os.path.join(
+            fig_dir, f"feat_dist_{title}_{feature_name}.{self.format}"
+        )
         if self.util.is_categorical(df_labels[label]):
             df_plot = pd.DataFrame(
                 {label: df_labels[label], feature: df_features[feature]}
             )
+            p_val = ""
+            if df_labels[label].nunique() == 2:
+                label_1 = df_labels[label].unique()[0]
+                label_2 = df_labels[label].unique()[1]
+                vals_1 = df_plot[df_plot[label] == label_1][feature].values
+                vals_2 = df_plot[df_plot[label] == label_2][feature].values
+                r_stats = stats.mannwhitneyu(vals_1, vals_2, alternative="two-sided")
+                p_val = f", Mann-Whitney p-val: {r_stats.pvalue:.3f}"
             ax = sns.violinplot(data=df_plot, x=label, y=feature)
             label = self.util.config_val("DATA", "target", "class_label")
-            ax.set(title=f"{title} samples", xlabel=label)
+            ax.set(title=f"{title} samples {p_val}", xlabel=label)
         else:
             plot_df = pd.concat([df_labels, df_features], axis=1)
             ax, caption = self._plot2cont(plot_df, label, feature, feature)
@@ -550,10 +653,9 @@ class Plots:
         # plt.tight_layout()
         # print(ax)
         # one up because of the runs
-        fig_dir = self.util.get_path("fig_dir") + "../"
+        fig_dir = os.path.join(self.util.get_path("fig_dir"), "..")
         exp_name = self.util.get_exp_name(only_data=True)
-        format = self.util.config_val("PLOT", "format", "png")
-        filename = f"{fig_dir}{exp_name}EXPL_tree-plot.{format}"
+        filename = os.path.join(fig_dir, f"{exp_name}EXPL_tree-plot.{self.format}")
         fig = ax.figure
         fig.savefig(filename)
         fig.clear()
@@ -561,8 +663,8 @@ class Plots:
         glob_conf.report.add_item(
             ReportItem(
                 Header.HEADER_EXPLORE,
-                f"Tree plot",
-                f"for feature importance",
+                "Tree plot",
+                "for feature importance",
                 filename,
             )
         )
