@@ -1,5 +1,4 @@
 class DefineBase:
-
     @classmethod
     def _assert_has_attribute_value(cls, value):
         valid_values = cls._attribute_values()
@@ -8,17 +7,16 @@ class DefineBase:
 
     @classmethod
     def _attribute_values(cls):
-        attributes = inspect.getmembers(
-            cls, lambda x: not inspect.isroutine(x)
-        )
+        attributes = inspect.getmembers(cls, lambda x: not inspect.isroutine(x))
         return sorted(
             [
-                a[1] for a in attributes
-                if not (a[0].startswith('__') and a[0].endswith('__'))
+                a[1]
+                for a in attributes
+                if not (a[0].startswith("__") and a[0].endswith("__"))
             ]
         )
 
 
 class Header(DefineBase):
-    HEADER_RESULTS = 'Results'
-    HEADER_EXPLORE = 'Data exploration'
+    HEADER_RESULTS = "Results"
+    HEADER_EXPLORE = "Data exploration"
