@@ -3,9 +3,9 @@ latex_writer.py
 print out report as latex file and pdf
 """
 
-from pylatex import Document, Section, Subsection, Command, Figure
-from pylatex.utils import italic, NoEscape
-from nkululeko.reporting.report_item import ReportItem
+from pylatex import Command, Document, Figure, Section, Subsection
+from pylatex.utils import NoEscape
+
 from nkululeko.utils.util import Util
 
 
