@@ -2,32 +2,30 @@ import ast
 import glob
 import json
 import math
-
-import matplotlib.pyplot as plt
-import numpy as np
-
-# from torch import is_tensor
-from audmetric import (
-    accuracy,
-    concordance_cc,
-    mean_absolute_error,
-    mean_squared_error,
-    unweighted_average_recall,
-)
+import os
 
 # import os
 from confidence_intervals import evaluate_with_conf_int
+import matplotlib.pyplot as plt
+import numpy as np
 from scipy.special import softmax
-from scipy.stats import entropy, pearsonr
-from sklearn.metrics import (
-    ConfusionMatrixDisplay,
-    auc,
-    classification_report,
-    confusion_matrix,
-    r2_score,
-    roc_auc_score,
-    roc_curve,
-)
+from scipy.stats import entropy
+from scipy.stats import pearsonr
+from sklearn.metrics import ConfusionMatrixDisplay
+from sklearn.metrics import RocCurveDisplay
+from sklearn.metrics import auc
+from sklearn.metrics import classification_report
+from sklearn.metrics import confusion_matrix
+from sklearn.metrics import r2_score
+from sklearn.metrics import roc_auc_score
+from sklearn.metrics import roc_curve
+
+# from torch import is_tensor
+from audmetric import accuracy
+from audmetric import concordance_cc
+from audmetric import mean_absolute_error
+from audmetric import mean_squared_error
+from audmetric import unweighted_average_recall
 
 import nkululeko.glob_conf as glob_conf
 from nkululeko.plots import Plots
@@ -75,6 +73,7 @@ class Reporter:
         self.result = Result(0, 0, 0, 0, "unknown")
         self.run = run
         self.epoch = epoch
+        self.model_type = self.util.get_model_type()
         self._set_metric()
         self.filenameadd = ""
         self.cont_to_cat = False
@@ -140,7 +139,7 @@ class Reporter:
             self.util.error(f"unknown metric: {self.metric}")
         return test_result, upper, lower
 
-    def print_probabilities(self):
+    def print_probabilities(self, file_name=None):
         """Print the probabilities per class to a file in the store."""
         if (
             self.util.exp_is_classification()
@@ -154,11 +153,14 @@ class Reporter:
             probas["truth"] = self.truths
             try:
                 le = glob_conf.label_encoder
-                mapping = dict(zip(le.classes_, range(len(le.classes_))))
-                mapping_reverse = {value: key for key, value in mapping.items()}
-                probas = probas.rename(columns=mapping_reverse)
-                probas["predicted"] = probas["predicted"].map(mapping_reverse)
-                probas["truth"] = probas["truth"].map(mapping_reverse)
+                if le is not None:
+                    mapping = dict(zip(le.classes_, range(len(le.classes_))))
+                    mapping_reverse = {value: key for key, value in mapping.items()}
+                    probas = probas.rename(columns=mapping_reverse)
+                    probas["predicted"] = probas["predicted"].map(mapping_reverse)
+                    probas["truth"] = probas["truth"].map(mapping_reverse)
+                else:
+                    self.util.debug("Label encoder is None, skipping label mapping")
             except AttributeError as ae:
                 self.util.debug(f"Can't label categories: {ae}")
             # compute entropy per sample
@@ -170,11 +172,22 @@ class Reporter:
             )
             probas["uncertainty"] = uncertainty
             probas["correct"] = probas.predicted == probas.truth
-            sp = self.util.get_pred_name()
-
+            if file_name is None:
+                file_name = self.util.get_pred_name() + ".csv"
+            else:
+                # Ensure the file_name goes to the results directory
+                if not os.path.isabs(file_name):
+                    res_dir = self.util.get_res_dir()
+                    if not file_name.endswith(".csv"):
+                        file_name = os.path.join(res_dir, file_name + ".csv")
+                    else:
+                        file_name = os.path.join(res_dir, file_name)
+                else:
+                    if not file_name.endswith(".csv"):
+                        file_name = file_name + ".csv"
             self.probas = probas
-            probas.to_csv(sp)
-            self.util.debug(f"Saved probabilities to {sp}")
+            probas.to_csv(file_name)
+            self.util.debug(f"Saved probabilities to {file_name}")
             plots = Plots()
             ax, caption = plots.plotcatcont(
                 probas, "correct", "uncertainty", "uncertainty", "correct"
@@ -184,7 +197,7 @@ class Reporter:
                 caption,
                 "Uncertainty",
                 "uncertainty_samples",
-                self.util.get_exp_name(),
+                "samples",
             )
 
     def set_id(self, run, epoch):
@@ -370,7 +383,7 @@ class Reporter:
 
         res_dir = self.util.get_path("res_dir")
         rpt = (
-            f"Best score at epoch: {epoch}, UAR: {uar_str}"
+            f"Confusion matrix result for epoch: {epoch}, UAR: {uar_str}"
             + f", (+-{up_str}/{low_str}), ACC: {acc_str}"
         )
         # print(rpt)
@@ -382,12 +395,30 @@ class Reporter:
     def set_filename_add(self, my_string):
         self.filenameadd = f"_{my_string}"
 
-    def print_results(self, epoch=None):
+    def print_logo(self, results):
+        res_dir = self.util.get_path("res_dir")
+        result_str = f"LOGO results: [{','.join(results.astype(str))}]"
+        file_name = f"{res_dir}/logo_results.txt"
+        with open(file_name, "w") as text_file:
+            text_file.write(
+                f"LOGO: mean {results.mean():.3f},  std: " + f"{results.std():.3f}"
+            )
+            text_file.write("\n")
+            text_file.write(result_str)
+        self.util.debug(result_str)
+
+    def print_results(self, epoch=None, file_name=None):
         if epoch is None:
             epoch = self.epoch
         """Print all evaluation values to text file."""
         res_dir = self.util.get_path("res_dir")
-        file_name = f"{res_dir}{self.util.get_exp_name()}_{epoch}{self.filenameadd}.txt"
+        if file_name is None:
+            file_name = (
+                f"{res_dir}{self.util.get_exp_name()}_{epoch}{self.filenameadd}.txt"
+            )
+        else:
+            self.util.debug(f"####->{file_name}<-####")
+            file_name = f"{res_dir}{file_name}{self.filenameadd}.txt"
         if self.util.exp_is_classification():
             labels = glob_conf.labels
             try:
@@ -397,6 +428,18 @@ class Reporter:
                     target_names=labels,
                     output_dict=True,
                 )
+                # in case we hade numbers before
+                s_labels = [str(x) for x in labels]
+                # print classifcation report in console
+                class_report_str = classification_report(
+                    self.truths,
+                    self.preds,
+                    target_names=s_labels,
+                    digits=4,
+                )
+                self.util.debug(
+                    f"\n {class_report_str}"
+                )
             except ValueError as e:
                 self.util.debug(
                     "Reporter: caught a ValueError when trying to get"
@@ -412,12 +455,6 @@ class Reporter:
                 f1_per_class = (
                     f"result per class (F1 score): {c_ress} from epoch: {epoch}"
                 )
-                if len(np.unique(self.truths)) == 2:
-                    fpr, tpr, _ = roc_curve(self.truths, self.preds)
-                    auc_score = auc(fpr, tpr)
-                    pauc_score = roc_auc_score(self.truths, self.preds, max_fpr=0.1)
-                    auc_pauc = f"auc: {auc_score:.3f}, pauc: {pauc_score:.3f} from epoch: {epoch}"
-                    self.util.debug(auc_pauc)
                 self.util.debug(f1_per_class)
                 rpt_str = f"{json.dumps(rpt)}\n{f1_per_class}"
                 # rpt_str += f"\n{auc_auc}"
@@ -486,18 +523,12 @@ class Reporter:
         # do a plot per run
         # scale the losses so they fit on the picture
         losses, results, train_results, losses_eval = (
-            np.asarray(losses),
-            np.asarray(results),
-            np.asarray(train_results),
-            np.asarray(losses_eval),
+            self._scaleresults(np.asarray(losses)),
+            self._scaleresults(np.asarray(results)),
+            self._scaleresults(np.asarray(train_results)),
+            self._scaleresults(np.asarray(losses_eval)),
         )
 
-        if np.all((results > 1)):
-            # scale down values
-            results = results / 100.0
-            train_results = train_results / 100.0
-        # if np.all((losses < 1)):
-        # scale up values
         plt.figure(dpi=200)
         plt.plot(train_results, "green", label="train set")
         plt.plot(results, "red", label="dev set")
@@ -508,3 +539,11 @@ class Reporter:
         plt.legend()
         plt.savefig(f"{fig_dir}{out_name}.{self.format}")
         plt.close()
+
+    def _scaleresults(self, results: np.ndarray) -> np.ndarray:
+        results = results.copy()
+        """Scale results to fit on the plot."""
+        if np.any((results > 1)):
+            # scale down values
+            results = results / 100.0
+        return results
