@@ -1,27 +1,48 @@
+"""Resample audio files or INI files (train, test, all) to change the sampling rate.
+
+This script provides a command-line interface to resample audio files or INI files
+containing train, test, and all data. It supports resampling a single file, a
+directory of files, or all files specified in an INI configuration file.
+
+The script uses the `Resampler` class from the `nkululeko.augmenting.resampler`
+module to perform the resampling operation. It can optionally replace the original
+audio files with the resampled versions.
+
+The script supports the following command-line arguments:
+- `--config`: The base configuration file (INI format) to use for resampling.
+- `--file`: The input audio file to resample.
+- `--folder`: The input directory containing audio files and subdirectories to resample.
+- `--replace`: Whether to replace the original audio files with the resampled versions.
+
+The script also supports loading configuration from an INI file, which can be used
+to specify the sample selection (all, train, or test) and whether to replace the
+original files.
+"""
+
 # resample.py
 # change the sampling rate for audio file or INI file (train, test, all)
 
 import argparse
 import configparser
 import os
-import pandas as pd
-import audformat
-from nkululeko.augmenting.resampler import Resampler
-from nkululeko.utils.util import Util
 
+import pandas as pd
+
+import audformat
+
+from nkululeko.augmenting.resampler import Resampler
 from nkululeko.constants import VERSION
 from nkululeko.experiment import Experiment
 from nkululeko.utils.files import find_files
+from nkululeko.utils.util import Util
 
 
-def main(src_dir):
+def main():
     parser = argparse.ArgumentParser(
         description="Call the nkululeko RESAMPLE framework."
     )
-    parser.add_argument("--config", default=None,
-                        help="The base configuration")
-    parser.add_argument("--file", default=None,
-                        help="The input audio file to resample")
+    parser.add_argument("--config", default=None, help="The base configuration")
+    parser.add_argument("--file", default=None, help="The input audio file to resample")
     parser.add_argument(
         "--folder",
         default=None,
@@ -34,9 +55,7 @@ def main(src_dir):
     args = parser.parse_args()
 
     if args.file is None and args.folder is None and args.config is None:
-        print(
-            "ERROR: Either --file, --folder, or --config argument must be provided."
-        )
+        print("ERROR: Either --file, --folder, or --config argument must be provided.")
         exit()
 
     if args.file is not None:
@@ -102,8 +121,7 @@ def main(src_dir):
             f"train shape : {expr.df_train.shape}, test shape:{expr.df_test.shape}"
         )
 
-        sample_selection = util.config_val(
-            "RESAMPLE", "sample_selection", "all")
+        sample_selection = util.config_val("RESAMPLE", "sample_selection", "all")
         if sample_selection == "all":
             df = pd.concat([expr.df_train, expr.df_test])
         elif sample_selection == "train":
@@ -122,5 +140,4 @@ def main(src_dir):
 
 
 if __name__ == "__main__":
-    cwd = os.path.dirname(os.path.abspath(__file__))
-    main(cwd)
+    main()
