@@ -1,6 +1,7 @@
 # util.py
 import ast
 import configparser
+import json
 import logging
 import os.path
 import pickle
@@ -27,9 +28,11 @@ class Util:
         "pkl",
         "eGeMAPSv02",
         "functionals",
+        "n_jobs",
     ]
 
     def __init__(self, caller=None, has_config=True):
+        self.logger = None
         if caller is not None:
             self.caller = caller
         else:
@@ -48,7 +51,12 @@ class Util:
                         self.error(f"no such file: {self.got_data_roots}")
                     self.data_roots = configparser.ConfigParser()
                     self.data_roots.read(self.got_data_roots)
-            except (ModuleNotFoundError, AttributeError):
+            except ModuleNotFoundError as e:
+                self.error(e)
+                self.config = None
+                self.got_data_roots = False
+            except AttributeError as e:
+                self.error(e)
                 self.config = None
                 self.got_data_roots = False
 
@@ -86,6 +94,8 @@ class Util:
                 dir_name = "./results/"
             elif entry == "model_dir":
                 dir_name = "./models/"
+            elif entry == "cache":
+                dir_name = "./cache/"
             else:
                 dir_name = "./store/"
         else:
@@ -96,13 +106,15 @@ class Util:
             except KeyError:
                 # some default values
                 if entry == "fig_dir":
-                    entryn = "./images/"
+                    entryn = "images/"
                 elif entry == "res_dir":
-                    entryn = "./results/"
+                    entryn = "results/"
                 elif entry == "model_dir":
-                    entryn = "./models/"
+                    entryn = "models/"
+                elif entry == "cache":
+                    entryn = "cache/"
                 else:
-                    entryn = "./store/"
+                    entryn = "store/"
 
             # Expand image, model and result directories with run index
             if entry == "fig_dir" or entry == "res_dir" or entry == "model_dir":
@@ -150,18 +162,33 @@ class Util:
         # self.logged_configs.clear()
 
     def get_save_name(self):
-        """Return a relative path to a name to save the experiment"""
+        """Return a relative path to a name to save the experiment."""
         store = self.get_path("store")
         return f"{store}/{self.get_exp_name()}.pkl"
 
     def get_pred_name(self):
-        store = self.get_path("store")
+        results_dir = self.get_path("res_dir")
         target = self.get_target_name()
         pred_name = self.get_model_description()
-        return f"{store}/pred_{target}_{pred_name}.csv"
+        return f"{results_dir}/pred_{target}_{pred_name}"
+
+    def print_results_to_store(self, name: str, contents: str) -> str:
+        """Write contents to a result file.
+
+        Args:
+            name (str): the (sub) name of the file_
+
+        Returns:
+            str: The path to the file
+        """
+        results_dir = self.get_path("res_dir")
+        pred_name = self.get_model_description()
+        path = os.path.join(results_dir, f"{name}_{pred_name}.txt")
+        with open(path, "a") as f:
+            f.write(contents)
 
     def is_categorical(self, pd_series):
-        """Check if a dataframe column is categorical"""
+        """Check if a dataframe column is categorical."""
         return pd_series.dtype.name == "object" or isinstance(
             pd_series.dtype, pd.CategoricalDtype
         )
@@ -174,7 +201,7 @@ class Util:
         """Get the experiment directory."""
         root = os.path.join(self.config["EXP"]["root"], "")
         name = self.config["EXP"]["name"]
-        dir_name = f"{root}{name}"
+        dir_name = f"{root}/{name}"
         audeer.mkdir(dir_name)
         return dir_name
 
@@ -226,11 +253,18 @@ class Util:
         return self.config["DATA"]["target"]
 
     def get_model_type(self):
-        return self.config["MODEL"]["type"]
+        try:
+            return self.config["MODEL"]["type"]
+        except KeyError:
+            return ""
 
     def get_model_description(self):
         mt = ""
-        mt = f'{self.config["MODEL"]["type"]}'
+        try:
+            mt = f'{self.config["MODEL"]["type"]}'
+        except KeyError:
+            # no model type given
+            pass
         # ft = "_".join(ast.literal_eval(self.config["FEATS"]["type"]))
         ft_value = self.config["FEATS"]["type"]
         if (
@@ -289,14 +323,23 @@ class Util:
         return False
 
     def error(self, message):
-        self.logger.error(f"ERROR: {self.caller}: {message}")
+        if self.logger is not None:
+            self.logger.error(f"ERROR: {self.caller}: {message}")
+        else:
+            print(f"ERROR: {message}")
         sys.exit()
 
     def warn(self, message):
-        self.logger.warning(f"WARNING: {self.caller}: {message}")
+        if self.logger is not None:
+            self.logger.warning(f"WARNING: {self.caller}: {message}")
+        else:
+            print(f"WARNING: {message}", flush=True)
 
     def debug(self, message):
-        self.logger.debug(f"DEBUG: {self.caller}: {message}")
+        if self.logger is not None:
+            self.logger.debug(f"DEBUG: {self.caller}: {message}")
+        else:
+            print(f"DEBUG: {message}", flush=True)
 
     def set_config_val(self, section, key, value):
         try:
@@ -307,7 +350,7 @@ class Util:
             self.config[section][key] = str(value)
 
     def check_df(self, i, df):
-        """Check a dataframe"""
+        """Check a dataframe."""
         print(f"check {i}: {df.shape}")
         print(df.head(1))
 
@@ -469,3 +512,28 @@ class Util:
     def to_3_digits_str(self, x):
         """Given a float, return this to 3 digits as string without integer number."""
         return str(self.to_3_digits(x))[1:]
+
+    def save_json(self, file: str, var: dict):
+        """Save variable to json file.
+
+        Args:
+            file: path to json file
+            var: dictionary to store
+
+        """
+        with open(file, "w", encoding="utf-8") as fp:
+            json.dump(var, fp, ensure_ascii=False, indent=2)
+
+    def read_json(self, file: str) -> object:
+        """Read variable from json file.
+
+        Args:
+            file: path to json file
+
+        Returns:
+            content of json file
+
+        """
+        with open(file, "r") as fp:
+            return json.load(fp)
+
